# @rickosborne/foundation

Basic data structures and algorithms.
***

## API

### Classes

#### ArrayBinaryMinHeap

<a id="api-arraybinaryminheap"></a>

```typescript
declare class ArrayBinaryMinHeap<T> implements Queue<T> 
```

#### PriorityArray

<a id="api-priorityarray"></a>

```typescript
declare class PriorityArray<T> implements Queue<T> 
```

#### PriorityLinkedList

<a id="api-prioritylinkedlist"></a>

```typescript
declare class PriorityLinkedList<T> implements Queue<T> 
```

#### SkipList

<a id="api-skiplist"></a>

```typescript
declare class SkipList<T> implements Queue<T> 
```

### Functions

#### addProperties

<a id="api-addproperties"></a>

```typescript
addProperties: <T, K extends string, R extends Record<K, TypedPropertyDescriptor<unknown>>>(target: T, descriptors: R) => Resolve<T & PropertiesFromDescriptors<R>>
```

A more type-friendly version of <kbd>Object.defineProperties</kbd>. See [Object.defineProperties on MDN](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Object/defineProperties).


#### addProperty

<a id="api-addproperty"></a>

```typescript
addProperty: <T, K extends string, V, D extends TypedPropertyDescriptor<V>>(target: T, key: K, descriptor: D) => Resolve<T & PropertyFromDescriptor<K, D>>
```

A more type-friendly version of <kbd>Object.defineProperty</kbd>. See [Object.defineProperty on MDN](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Object/defineProperty).


#### arrayBinaryMaxHeap

<a id="api-arraybinarymaxheap"></a>

```typescript
arrayBinaryMaxHeap: <T>(comparator: Comparator<T>) => ArrayBinaryMinHeap<T>
```

#### arrayBinaryMinHeap

<a id="api-arraybinaryminheap"></a>

```typescript
arrayBinaryMinHeap: <T>(comparator: Comparator<T>) => ArrayBinaryMinHeap<T>
```

#### binaryIndexOf

<a id="api-binaryindexof"></a>

```typescript
binaryIndexOf: <T>(value: T, items: T[], comparator: Comparator<T>, { initialLeft, initialRight, firstNonUnique, rangeCheck }?: BinaryIndexOfConfig) => SearchResult
```

#### chainableComparator

<a id="api-chainablecomparator"></a>

```typescript
chainableComparator: <T>(comparator: Comparator<T>) => ChainableComparator<T>
```

#### comparatorBuilder

<a id="api-comparatorbuilder"></a>

```typescript
comparatorBuilder: <T>() => ComparatorBuilder<T>
```

#### deepSort

<a id="api-deepsort"></a>

```typescript
deepSort: <T>(obj: T) => T
```

Sort an object, and any objects nested through plain objects or arrays, by key, so when JSON stringified you will get a deterministic output.


#### doubleLinkedList

<a id="api-doublelinkedlist"></a>

```typescript
doubleLinkedList: <T>() => DoubleLinkedList<T>
```

#### priorityArray

<a id="api-priorityarray"></a>

```typescript
priorityArray: <T>(comparator: Comparator<T>) => Queue<T>
```

#### priorityLinkedList

<a id="api-prioritylinkedlist"></a>

```typescript
priorityLinkedList: <T>(comparator: Comparator<T>) => Queue<T>
```

#### shuffle

<a id="api-shuffle"></a>

```typescript
shuffle: <T>(list: T[], { into, random01 }?: ShuffleConfig<T>) => T[]
```

Fisher-Yates shuffle. Defaults to in-place sorting, but can sort into a new one via <kbd>into</kbd>.



#### singleLinkedList

<a id="api-singlelinkedlist"></a>

```typescript
singleLinkedList: <T>() => List<T>
```

#### skipList

<a id="api-skiplist"></a>

```typescript
skipList: <T>(comparator: Comparator<T>) => SkipList<T>
```

#### toShuffled

<a id="api-toshuffled"></a>

```typescript
toShuffled: <T>(list: T[], config?: Omit<ShuffleConfig<T>, "into"> | undefined) => T[]
```

#### withNullsFirst

<a id="api-withnullsfirst"></a>

```typescript
withNullsFirst: <T>(comparator: Comparator<T>) => Comparator<T | null | undefined>
```

#### withNullsLast

<a id="api-withnullslast"></a>

```typescript
withNullsLast: <T>(comparator: Comparator<T>) => Comparator<T | null | undefined>
```

### TypeAliases

#### BinaryIndexOfConfig

<a id="api-binaryindexofconfig"></a>

```typescript
type BinaryIndexOfConfig = {
    firstNonUnique?: boolean;
    initialLeft?: number;
    initialRight?: number;
    rangeCheck?: boolean;
};
```

#### ChainableComparator

<a id="api-chainablecomparator"></a>

```typescript
type ChainableComparator<T> = Comparator<T> & {
    get desc(): ChainableComparator<T>;
    get nullsFirst(): ChainableComparator<T | undefined | null>;
    get nullsLast(): ChainableComparator<T | undefined | null>;
};
```

#### ComparatorBuilder

<a id="api-comparatorbuilder"></a>

```typescript
type ComparatorBuilder<T> = {
    build(): Comparator<T>;
    get desc(): ComparatorBuilder<T>;
    get nullsFirst(): ComparatorBuilder<T>;
    get nullsLast(): ComparatorBuilder<T>;
    num(accessor: (t: T) => (undefined | number)): ComparatorBuilder<T>;
    str(accessor: (t: T) => (undefined | string)): ComparatorBuilder<T>;
};
```

#### DoubleLinkedList

<a id="api-doublelinkedlist"></a>

```typescript
type DoubleLinkedList<T> = List<T> & {
    get tail(): T | undefined;
    reverseValues(): Generator<T, void, undefined>;
    reverseValuesAndIndexes(): Generator<[T, number], void, undefined>;
    reverseWalk(block: (value: T, index: number) => ListWalkDecision<T>): void;
};
```

#### GetPropertyDescriptor

<a id="api-getpropertydescriptor"></a>

```typescript
type GetPropertyDescriptor<T> = PropertyDescriptorFlags & {
    get(): T;
    set?: undefined;
};
```

Partial property descriptor which includes only an accessor (getter) and not a mutator, and is thus read-only.


#### GetSetPropertyDescriptor

<a id="api-getsetpropertydescriptor"></a>

```typescript
type GetSetPropertyDescriptor<T> = PropertyDescriptorFlags & {
    get(): T;
    set(t: T): void;
};
```

Partial property descriptor which includes both an accessor (getter) and mutator (setter), and is thus read-write.


#### IfReadOnlyDescriptor

<a id="api-ifreadonlydescriptor"></a>

```typescript
type IfReadOnlyDescriptor<T, D extends TypedPropertyDescriptor<unknown>> = D extends ReadOnlyPropertyDescriptor | GetPropertyDescriptor<unknown> ? T : never;
```

Filter type <kbd>T</kbd> based on whether the property descriptor <kbd>D</kbd> would produce a read-only property.


#### IfReadWriteDescriptor

<a id="api-ifreadwritedescriptor"></a>

```typescript
type IfReadWriteDescriptor<T, D extends TypedPropertyDescriptor<unknown>> = D extends ReadOnlyPropertyDescriptor | GetPropertyDescriptor<unknown> ? never : T;
```

Filter type <kbd>T</kbd> based on whether the property descriptor <kbd>D</kbd> would produce a read-write (or write-only) property.


#### IfWriteOnlyDescriptor

<a id="api-ifwriteonlydescriptor"></a>

```typescript
type IfWriteOnlyDescriptor<T, D extends TypedPropertyDescriptor<unknown>> = D extends SetPropertyDescriptor<unknown> ? T : never;
```

Filter type <kbd>T</kbd> based on whether the property descriptor <kbd>D</kbd> would produce a write-only property. Note: JS has a concept of a write-only property, and the unit tests cover this. However, TS doesn't have a way of expressing it, at least as of v5.8. So for now, it goes into the read-write bucket, even though you can't actually read from it. The alternative would be to set it to ` undefined `, to show you can't read from it ... but then you lose the type info for the setter. Yuck. For now ... The types below will, ahem, mangle the types they produce. Just a little. If you produce a setter-only property, it will end up looking like:

```typescript
const withSetterOnly: WithSetterOnly = addProperty({}, "whatever", { set: (value: string) => void(whatever(value)) });
type WithSetterOnly = {
   whatever: string;
   whateverIsWriteOnly?: undefined;
}
```


So you get a reminder without breaking anything too hard.


#### IntGenerator

<a id="api-intgenerator"></a>

```typescript
type IntGenerator = Generator<number, void, undefined> & {
    toArray(): number[];
};
```

Mixin for a generator which also has the ability to bundle its output into an array.


#### IntGeneratorTo

<a id="api-intgeneratorto"></a>

```typescript
type IntGeneratorTo = {
    toExclusive(stopExclusive: number): IntGenerator;
    toInclusive(stopInclusive: number): IntGenerator;
};
```

Partial for the last step of an [IntRange](#api-intrange) builder.


#### IntRange

<a id="api-intrange"></a>

```typescript
type IntRange = {
    from(startInclusive: number): {
        by(skip: number): IntGeneratorTo;
        get down(): IntGeneratorTo;
        get up(): IntGeneratorTo;
    };
};
```

Integer range generators which (hopefully) make it super clear what your boundary, direction, and increment are.


#### List

<a id="api-list"></a>

```typescript
type List<T> = {
    at(index: number): T | undefined;
    get head(): T | undefined;
    insertAt(index: number, value: T): void;
    get isEmpty(): boolean;
    get length(): number;
    pop(): T | undefined;
    push(value: T): void;
    shift(): T | undefined;
    toArray(): T[];
    unshift(value: T): void;
    values(): Generator<T, void, undefined>;
    valuesAndIndexes(): Generator<[T, number], void, undefined>;
    walk(block: (value: T, index: number) => ListWalkDecision<T>): void;
    readonly [Symbol.toStringTag]: string;
};
```

#### ListWalkDecision

<a id="api-listwalkdecision"></a>

```typescript
type ListWalkDecision<T> = {
    deleteItem?: boolean;
    insertAfter?: T;
    insertBefore?: T;
    keepWalking: boolean;
};
```

#### PropertiesFromDescriptors

<a id="api-propertiesfromdescriptors"></a>

```typescript
type PropertiesFromDescriptors<R extends Record<string, TypedPropertyDescriptor<unknown>>> = {
    readonly [K in ReadOnlyDescriptorsKeys<R>]: R[K] extends TypedPropertyDescriptor<infer V> ? V : never;
} & {
    [K in ReadWriteDescriptorsKeys<R>]: R[K] extends TypedPropertyDescriptor<infer V> ? V : never;
} & {
    [K in RenamedWriteOnlyDescriptorsKeys<R>]?: undefined;
};
```

Produce an object with the properties described by the descriptors. See [RenameIfWriteOnlyDescriptor](#api-renameifwriteonlydescriptor) for details on the mangling of write-only properties.


#### PropertyDescriptorFlags

<a id="api-propertydescriptorflags"></a>

```typescript
type PropertyDescriptorFlags = {
    configurable?: boolean;
    enumerable?: boolean;
    writable?: boolean;
};
```

Basic flags for any property descriptor.


#### PropertyFromDescriptor

<a id="api-propertyfromdescriptor"></a>

```typescript
type PropertyFromDescriptor<K extends string, D extends TypedPropertyDescriptor<unknown>> = D extends TypedPropertyDescriptor<infer V> ? ({
    readonly [key in IfReadOnlyDescriptor<K, D>]: V;
} & {
    [key in IfReadWriteDescriptor<K, D>]: V;
} & {
    [key in RenameIfWriteOnlyDescriptor<K, D>]?: undefined;
}) : never;
```

Produce a single-property object type for the given property descriptor. Note: will produce an additional mangled pseudo-property for write-only properties. See [RenameIfWriteOnlyDescriptor](#api-renameifwriteonlydescriptor) for details on the mangling.


#### Queue

<a id="api-queue"></a>

```typescript
type Queue<T> = {
    add(value: T): void;
    peek(): T | undefined;
    remove(value: T): void;
    get length(): number;
    take(): T | undefined;
    toArray(): T[];
    values(): Generator<T, void, undefined>;
};
```

#### ReadOnlyDescriptorsKeys

<a id="api-readonlydescriptorskeys"></a>

```typescript
type ReadOnlyDescriptorsKeys<R> = R extends Record<infer K, TypedPropertyDescriptor<unknown>> ? {
    [key in K]: IfReadOnlyDescriptor<key, R[key]>;
}[K] : never;
```

Produce the keys for the given property descriptors record which would produce read-only properties.


#### ReadOnlyPropertyDescriptor

<a id="api-readonlypropertydescriptor"></a>

```typescript
type ReadOnlyPropertyDescriptor = {
    writable: false;
};
```

Partial property descriptor which has been marked as non-writable.


#### ReadWriteDescriptorsKeys

<a id="api-readwritedescriptorskeys"></a>

```typescript
type ReadWriteDescriptorsKeys<R> = R extends Record<infer K, TypedPropertyDescriptor<unknown>> ? {
    [key in K]: IfReadWriteDescriptor<key, R[key]>;
}[K] : never;
```

Produce the keys for the given property descriptors record which would produce read-write (or write-only) properties.


#### RenamedWriteOnlyDescriptorsKeys

<a id="api-renamedwriteonlydescriptorskeys"></a>

```typescript
type RenamedWriteOnlyDescriptorsKeys<R> = R extends Record<infer K, TypedPropertyDescriptor<unknown>> ? {
    [key in string & K]: RenameIfWriteOnlyDescriptor<key, R[key]>;
}[string & K] : never;
```

Produce the <strong>mangled</strong> keys for the given property descriptors record which would produce write-only properties. See [RenameIfWriteOnlyDescriptor](#api-renameifwriteonlydescriptor) for details on the mangling.


#### RenameIfWriteOnlyDescriptor

<a id="api-renameifwriteonlydescriptor"></a>

```typescript
type RenameIfWriteOnlyDescriptor<Key extends string, D extends TypedPropertyDescriptor<unknown>> = D extends SetPropertyDescriptor<unknown> ? `${Key}IsWriteOnly` : never;
```

Mangle the given <kbd>Key</kbd> to <kbd>$\{Key\}IsWriteOnly</kbd> if property descriptor <kbd>D</kbd> would produce a write-only property.


#### SearchResult

<a id="api-searchresult"></a>

```typescript
type SearchResult = {
    before?: undefined;
    comparisons: number;
    exists: true;
    index: number;
} | {
    before: number;
    comparisons: number;
    exists: false;
    index?: undefined;
};
```

#### SetPropertyDescriptor

<a id="api-setpropertydescriptor"></a>

```typescript
type SetPropertyDescriptor<T> = PropertyDescriptorFlags & {
    get?: undefined;
    set(t: T): void;
};
```

Partial property descriptor which includes only a mutator (setter) and not an accessor (getter), and is thus write-only. Note: TypeScript support for these kinds of properties is very poor as of v5.7.


#### ShuffleConfig

<a id="api-shuffleconfig"></a>

```typescript
type ShuffleConfig<T> = {
    into?: T[] | undefined;
    random01?: () => number;
};
```

#### TypedPropertyDescriptor_2

<a id="api-typedpropertydescriptor-2"></a>

```typescript
type TypedPropertyDescriptor<T> = ValuePropertyDescriptor<T> | GetPropertyDescriptor<T> | SetPropertyDescriptor<T> | GetSetPropertyDescriptor<T>;
```

Union for reasonable configurations of property descriptors which could affect the type for the property.


#### ValuePropertyDescriptor

<a id="api-valuepropertydescriptor"></a>

```typescript
type ValuePropertyDescriptor<T> = PropertyDescriptorFlags & {
    value: T;
};
```

Partial property descriptor which contains a static value, instead of an accessor and/or mutator.


### Variables

#### intRange

<a id="api-intrange"></a>

```typescript
intRange: IntRange
```

#### numberAsc

<a id="api-numberasc"></a>

```typescript
numberAsc: Comparator<number>
```

