"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.comparatorBuilder = exports.numberAsc = exports.chainableComparator = exports.withNullsLast = exports.withNullsFirst = void 0;
const guard_1 = require("@rickosborne/guard");
const withNullsFirst = (comparator) => {
    return function nullsFirst(a, b) {
        if (a == null && b == null) {
            return 0;
        }
        else if (a == null) {
            return -1;
        }
        else if (b == null) {
            return 1;
        }
        return comparator(a, b);
    };
};
exports.withNullsFirst = withNullsFirst;
const withNullsLast = (comparator) => {
    return function nullsLast(a, b) {
        if (a == null && b == null) {
            return 0;
        }
        else if (a == null) {
            return 1;
        }
        else if (b == null) {
            return -1;
        }
        return comparator(a, b);
    };
};
exports.withNullsLast = withNullsLast;
const withDesc = (comparator) => function desc(a, b) {
    return comparator(b, a);
};
const chainableComparator = (comparator) => {
    let _nullsFirst;
    let _nullsLast;
    let _desc;
    const added = ["nullsFirst", "nullsLast", "desc"];
    return new Proxy(comparator, {
        get(target, p) {
            if (p === "nullsFirst") {
                _nullsFirst !== null && _nullsFirst !== void 0 ? _nullsFirst : (_nullsFirst = (0, exports.chainableComparator)((0, exports.withNullsFirst)(comparator)));
                return _nullsFirst;
            }
            if (p === "nullsLast") {
                _nullsLast !== null && _nullsLast !== void 0 ? _nullsLast : (_nullsLast = (0, exports.chainableComparator)((0, exports.withNullsLast)(comparator)));
                return _nullsLast;
            }
            if (p === "desc") {
                _desc !== null && _desc !== void 0 ? _desc : (_desc = (0, exports.chainableComparator)(withDesc(comparator)));
                return _desc;
            }
            return target[p];
        },
        getOwnPropertyDescriptor(target, p) {
            if (added.includes(p)) {
                return {
                    enumerable: true,
                    get: () => target[p],
                };
            }
            return Object.getOwnPropertyDescriptor(target, p);
        },
        has(target, p) {
            return added.includes(p) || Object.hasOwn(target, p);
        },
        ownKeys(target) {
            return [...added, ...Object.keys(target)];
        },
    });
};
exports.chainableComparator = chainableComparator;
const numberAsc = (a, b) => a - b;
exports.numberAsc = numberAsc;
const comparatorBuilder = () => {
    const ops = [];
    const withPrevious = (modifier) => {
        const lastIndex = ops.length - 1;
        const comparator = ops[lastIndex][1];
        (0, guard_1.assertDefined)(comparator, "Prior comparator");
        ops[lastIndex][1] = modifier(comparator);
    };
    const builder = {
        build() {
            return (a, b) => {
                for (const [accessor, comparator] of ops) {
                    const aValue = accessor(a);
                    const bValue = accessor(b);
                    const result = comparator(aValue, bValue);
                    if (result !== 0) {
                        return result;
                    }
                }
                return 0;
            };
        },
        get desc() {
            withPrevious((c) => withDesc(c));
            return builder;
        },
        get nullsFirst() {
            withPrevious((c) => (0, exports.withNullsFirst)(c));
            return builder;
        },
        get nullsLast() {
            withPrevious((c) => (0, exports.withNullsLast)(c));
            return builder;
        },
        num(accessor) {
            ops.push([accessor, (a, b) => {
                    if (a == null && b == null) {
                        return 0;
                    }
                    if (a == null) {
                        return 1;
                    }
                    if (b == null) {
                        return -1;
                    }
                    return a - b;
                }]);
            return builder;
        },
        str(accessor) {
            ops.push([accessor, (a, b) => {
                    if (a == null && b == null) {
                        return 0;
                    }
                    if (a == null) {
                        return 1;
                    }
                    if (b == null) {
                        return -1;
                    }
                    return a.localeCompare(b);
                }]);
            return builder;
        },
    };
    return builder;
};
exports.comparatorBuilder = comparatorBuilder;
//# sourceMappingURL=comparator.js.map