"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.toShuffled = exports.shuffle = void 0;
/**
 * Fisher-Yates shuffle.
 * Defaults to in-place sorting, but can sort into a new one via <kbd>into</kbd>.
 * @see {@link https://en.wikipedia.org/wiki/Fisher%E2%80%93Yates_shuffle | Fisher-Yates shuffle on Wikipedia}
 */
const shuffle = (list, { into = list, random01 = Math.random } = {}) => {
    const n = list.length;
    for (let i = 0; i < n; i++) {
        const j = Math.trunc(random01() * (i + 1));
        const t = list[i];
        into[i] = into[j];
        into[j] = t;
    }
    return into;
};
exports.shuffle = shuffle;
const toShuffled = (list, config) => (0, exports.shuffle)(list, Object.assign(Object.assign({}, config), { into: Array(list.length) }));
exports.toShuffled = toShuffled;
//# sourceMappingURL=shuffle.js.map