import { hasOwn } from "@rickosborne/guard";
import { addProperty } from "./add-property.js";
export const intRange = {
    from(startInclusive) {
        let skipBy = 1;
        let stop;
        let inclusive = false;
        const generatorFn = function* () {
            let more;
            if (skipBy < 0) {
                more = inclusive ? ((n) => n >= stop) : ((n) => n > stop);
            }
            else {
                more = inclusive ? ((n) => n <= stop) : ((n) => n < stop);
            }
            for (let i = startInclusive; more(i); i += skipBy) {
                yield i;
            }
        };
        const addToArray = (gen) => {
            if (hasOwn(gen, "toArray", (f) => typeof f === "function")) {
                return gen;
            }
            return addProperty(gen, "toArray", {
                configurable: false,
                enumerable: false,
                value: () => Array.from(generatorFn()),
                writable: false,
            });
        };
        const toExclusive = (stopExclusive) => {
            stop = stopExclusive;
            inclusive = false;
            return addToArray(generatorFn());
        };
        const toInclusive = (stopInclusive) => {
            stop = stopInclusive;
            inclusive = true;
            return addToArray(generatorFn());
        };
        const by = (skip) => {
            skipBy = skip;
            return { toExclusive, toInclusive };
        };
        return {
            by,
            get down() {
                skipBy = -1;
                return { toExclusive, toInclusive };
            },
            get up() {
                skipBy = 1;
                return { toExclusive, toInclusive };
            },
        };
    },
};
//# sourceMappingURL=int-range.js.map