/**
 * Mixin for a generator which also has the ability to bundle
 * its output into an array.
 */
export type IntGenerator = Generator<number, void, undefined> & {
    toArray(): number[];
};
/**
 * Partial for the last step of an {@link IntRange} builder.
 */
export type IntGeneratorTo = {
    /**
     * End before this number.
     */
    toExclusive(stopExclusive: number): IntGenerator;
    /**
     * End after this number.
     */
    toInclusive(stopInclusive: number): IntGenerator;
};
/**
 * Integer range generators which (hopefully) make it super clear what
 * your boundary, direction, and increment are.
 */
export type IntRange = {
    /**
     * Start with the given number, inclusively.
     */
    from(startInclusive: number): {
        /**
         * Increment/decrement by this number.
         */
        by(skip: number): IntGeneratorTo;
        /**
         * Decrement by 1.
         */
        get down(): IntGeneratorTo;
        /**
         * Increment by 1.
         */
        get up(): IntGeneratorTo;
    };
};
export declare const intRange: IntRange;
//# sourceMappingURL=int-range.d.ts.map