export type ShuffleConfig<T> = {
    /**
     * Target array.  Defaults to the given array, shuffling in place.
     */
    into?: T[] | undefined;
    /**
     * Generate a random float in the range <kbd>[0..1)</kbd>.
     */
    random01?: () => number;
};
/**
 * Fisher-Yates shuffle.
 * Defaults to in-place sorting, but can sort into a new one via <kbd>into</kbd>.
 * @see {@link https://en.wikipedia.org/wiki/Fisher%E2%80%93Yates_shuffle | Fisher-Yates shuffle on Wikipedia}
 */
export declare const shuffle: <T>(list: T[], { into, random01 }?: ShuffleConfig<T>) => T[];
export declare const toShuffled: <T>(list: T[], config?: Omit<ShuffleConfig<T>, "into"> | undefined) => T[];
//# sourceMappingURL=shuffle.d.ts.map